@@ -365,6 +365,7 @@ ld::File* InputFiles::makeFile(const Options::FileInfo& info, bool indirectDylib
 
 	ld::archive::File* archiveResult = ::archive::parse(p, len, info.path, info.modTime, info.ordinal, archOpts);
 	if ( archiveResult != NULL ) {
+	
 		OSAtomicAdd64(len, &_totalArchiveSize);
 		OSAtomicIncrement32(&_totalArchivesLoaded);
 		return archiveResult;
@@ -417,7 +418,7 @@ ld::File* InputFiles::makeFile(const Options::FileInfo& info, bool indirectDylib
 	}
 }
 
-void InputFiles::logDylib(ld::File* file, bool indirect)
+void InputFiles::logDylib(ld::File* file, bool indirect, bool speculative)
 {
 	if ( _options.traceDylibs() ) {
 		const char* fullPath = file->path();
@@ -429,11 +430,19 @@ void InputFiles::logDylib(ld::File* file, bool indirect)
 			// don't log upward dylibs when XBS is computing dependencies
 			logTraceInfo("[Logging for XBS] Used upward dynamic library: %s\n", fullPath);
 		}
+		else if ( (dylib != NULL ) && dylib->speculativelyLoaded() ) {
+			logTraceInfo("[Logging for XBS] Speculatively loaded dynamic library: %s\n", fullPath);
+		}
 		else {
-			if ( indirect ) 
-				logTraceInfo("[Logging for XBS] Used indirect dynamic library: %s\n", fullPath);
-			else 
+			if ( indirect ) {
+				if ( speculative )
+					logTraceInfo("[Logging for XBS] Speculatively loaded indirect dynamic library: %s\n", fullPath);
+				else
+					logTraceInfo("[Logging for XBS] Used indirect dynamic library: %s\n", fullPath);
+			}
+			else {
 				logTraceInfo("[Logging for XBS] Used dynamic library: %s\n", fullPath);
+			}
 		}
 	}
 	
@@ -459,7 +468,7 @@ void InputFiles::logDylib(ld::File* file, bool indirect)
 
 void InputFiles::logArchive(ld::File* file) const
 {
-	if ( _options.traceArchives() && (_archiveFilesLogged.count(file) == 0) ) {
+	if ( (_options.traceArchives() || _options.traceEmitJSON()) && (_archiveFilesLogged.count(file) == 0) ) {
 		// <rdar://problem/4947347> LD_TRACE_ARCHIVES should only print out when a .o is actually used from an archive
 		_archiveFilesLogged.insert(file);
 		const char* fullPath = file->path();
@@ -467,6 +476,9 @@ void InputFiles::logArchive(ld::File* file) const
 		if ( realpath(fullPath, realName) != NULL )
 			fullPath = realName;
 		logTraceInfo("[Logging for XBS] Used static archive: %s\n", fullPath);
+		
+		std::string archivePath(fullPath);
+		_archiveFilePaths.push_back(archivePath);
 	}
 }
 
@@ -505,7 +517,7 @@ void InputFiles::logTraceInfo(const char* format, ...) const
 }
 
 
-ld::dylib::File* InputFiles::findDylib(const char* installPath, const char* fromPath)
+ld::dylib::File* InputFiles::findDylib(const char* installPath, const ld::dylib::File* fromDylib, bool speculative)
 {
 	//fprintf(stderr, "findDylib(%s, %s)\n", installPath, fromPath);
 	InstallNameToDylib::iterator pos = _installPathToDylibs.find(installPath);
@@ -526,7 +538,7 @@ ld::dylib::File* InputFiles::findDylib(const char* installPath, const char* from
 					if ( dylibReader != NULL ) {
 						addDylib(dylibReader, info);
 						//_installPathToDylibs[strdup(installPath)] = dylibReader;
-						this->logDylib(dylibReader, true);
+						this->logDylib(dylibReader, true, speculative);
 						return dylibReader;
 					}
 					else 
@@ -537,20 +549,9 @@ ld::dylib::File* InputFiles::findDylib(const char* installPath, const char* from
 				}
 			}
 		}
-		char newPath[MAXPATHLEN];
-		// handle @loader_path
-		if ( strncmp(installPath, "@loader_path/", 13) == 0 ) {
-			strcpy(newPath, fromPath);
-			char* addPoint = strrchr(newPath,'/');
-			if ( addPoint != NULL )
-				strcpy(&addPoint[1], &installPath[13]);
-			else
-				strcpy(newPath, &installPath[13]);
-			installPath = newPath;
-		}
-		// note: @executable_path case is handled inside findFileUsingPaths()
-		// search for dylib using -F and -L paths
-		Options::FileInfo info = _options.findFileUsingPaths(installPath);
+
+		// search for dylib using -F and -L paths and expanding @ paths
+		Options::FileInfo info = _options.findIndirectDylib(installPath, fromDylib);
 		_indirectDylibOrdinal = _indirectDylibOrdinal.nextIndirectDylibOrdinal();
 		info.ordinal = _indirectDylibOrdinal;
 		info.options.fIndirectDylib = true;
@@ -561,7 +562,7 @@ ld::dylib::File* InputFiles::findDylib(const char* installPath, const char* from
 				//assert(_installPathToDylibs.find(installPath) !=  _installPathToDylibs.end());
 				//_installPathToDylibs[strdup(installPath)] = dylibReader;
 				addDylib(dylibReader, info);
-				this->logDylib(dylibReader, true);
+				this->logDylib(dylibReader, true, speculative);
 				return dylibReader;
 			}
 			else 
@@ -614,80 +615,96 @@ bool InputFiles::libraryAlreadyLoaded(const char* path)
 			return true;
 	}
 
+	char realDylibPath[PATH_MAX];
+	if ( (realpath(path, realDylibPath) != NULL) && (strcmp(path, realDylibPath) != 0) ) {
+		return libraryAlreadyLoaded(realDylibPath);
+	}
+
 	return false;
 }
 
 
 void InputFiles::addLinkerOptionLibraries(ld::Internal& state, ld::File::AtomHandler& handler)
 {	
-    if ( _options.outputKind() == Options::kObjectFile ) 
-		return;
+  	if ( _options.outputKind() == Options::kObjectFile )
+ 		return;
+  
+	while (! state.unprocessedLinkerOptionLibraries.empty() || ! state.unprocessedLinkerOptionFrameworks.empty()) {
 
-	// process frameworks specified in .o linker options
-	for (CStringSet::const_iterator it = state.linkerOptionFrameworks.begin(); it != state.linkerOptionFrameworks.end(); ++it) {
-		const char* frameworkName = *it;
-		if ( state.linkerOptionFrameworksProcessed.count(frameworkName) )
-			continue;
-		Options::FileInfo info = _options.findFramework(frameworkName);
-		if ( ! this->frameworkAlreadyLoaded(info.path, frameworkName) ) {
-			info.ordinal = _linkerOptionOrdinal.nextLinkerOptionOrdinal();
-			try {
-				ld::File* reader = this->makeFile(info, true);
-				ld::dylib::File* dylibReader = dynamic_cast<ld::dylib::File*>(reader);
-				if ( dylibReader != NULL ) {
-					if ( ! dylibReader->installPathVersionSpecific() ) {
+		// process frameworks specified in .o linker options
+		CStringSet newFrameworks = std::move(state.unprocessedLinkerOptionFrameworks);
+		state.unprocessedLinkerOptionFrameworks.clear();
+		for (const char* frameworkName : newFrameworks) {
+			if ( state.linkerOptionFrameworks.count(frameworkName) )
+				continue;
+			Options::FileInfo info = _options.findFramework(frameworkName);
+			if ( ! this->frameworkAlreadyLoaded(info.path, frameworkName) ) {
+				_linkerOptionOrdinal = _linkerOptionOrdinal.nextLinkerOptionOrdinal();
+				info.ordinal = _linkerOptionOrdinal;
+				try {
+					ld::File* reader = this->makeFile(info, true);
+					ld::dylib::File* dylibReader = dynamic_cast<ld::dylib::File*>(reader);
+					if ( dylibReader != NULL ) {
+						if ( ! dylibReader->installPathVersionSpecific() ) {
+							dylibReader->forEachAtom(handler);
+							dylibReader->setImplicitlyLinked();
+							dylibReader->setSpeculativelyLoaded();
+							this->addDylib(dylibReader, info);
+						}
+					}
+					else {
+						throwf("framework linker option at %s is not a dylib", info.path);
+ 					}
+ 				}
+				catch (const char* msg) {
+					warning("Auto-Linking supplied '%s', %s", info.path, msg);
+ 				}
+ 			}
+			state.linkerOptionFrameworks.insert(frameworkName);
+ 		}
+
+		// process libraries specified in .o linker options
+		// fixme optimize with std::move?
+		CStringSet newLibraries = std::move(state.unprocessedLinkerOptionLibraries);
+		state.unprocessedLinkerOptionLibraries.clear();
+		for (const char* libName : newLibraries) {
+			if ( state.linkerOptionLibraries.count(libName) )
+				continue;
+			Options::FileInfo info = _options.findLibrary(libName);
+			if ( ! this->libraryAlreadyLoaded(info.path) ) {
+				_linkerOptionOrdinal = _linkerOptionOrdinal.nextLinkerOptionOrdinal();
+				info.ordinal = _linkerOptionOrdinal;
+				try {
+ 					//<rdar://problem/17787306> -force_load_swift_libs
+					info.options.fForceLoad = _options.forceLoadSwiftLibs() && (strncmp(libName, "swift", 5) == 0);
+					ld::File* reader = this->makeFile(info, true);
+					ld::dylib::File* dylibReader = dynamic_cast<ld::dylib::File*>(reader);
+					ld::archive::File* archiveReader = dynamic_cast<ld::archive::File*>(reader);
+					if ( dylibReader != NULL ) {
 						dylibReader->forEachAtom(handler);
 						dylibReader->setImplicitlyLinked();
+						dylibReader->setSpeculativelyLoaded();
 						this->addDylib(dylibReader, info);
 					}
-				}
-				else {
-					throwf("framework linker option at %s is not a dylib", info.path);
-				}
-			}
-			catch (const char* msg) {
-				warning("Auto-Linking supplied '%s', %s", info.path, msg);
-			}
-		}
-		state.linkerOptionFrameworksProcessed.insert(frameworkName);
-	}
-	// process libraries specified in .o linker options
-	for (CStringSet::const_iterator it = state.linkerOptionLibraries.begin(); it != state.linkerOptionLibraries.end(); ++it) {
-		const char* libName = *it;
-		if ( state.linkerOptionLibrariesProcessed.count(libName) )
-			continue;
-		Options::FileInfo info = _options.findLibrary(libName);
-		if ( ! this->libraryAlreadyLoaded(info.path) ) {
-			info.ordinal = _linkerOptionOrdinal.nextLinkerOptionOrdinal();
-			try {
-				//<rdar://problem/17787306> -force_load_swift_libs
-				info.options.fForceLoad = _options.forceLoadSwiftLibs() && (strncmp(libName, "swift", 5) == 0);
-				ld::File* reader = this->makeFile(info, true);
-				ld::dylib::File* dylibReader = dynamic_cast<ld::dylib::File*>(reader);
-				ld::archive::File* archiveReader = dynamic_cast<ld::archive::File*>(reader);
-				if ( dylibReader != NULL ) {
-					dylibReader->forEachAtom(handler);
-					dylibReader->setImplicitlyLinked();
-					this->addDylib(dylibReader, info);
-				}
-				else if ( archiveReader != NULL ) {
-					_searchLibraries.push_back(LibraryInfo(archiveReader));
-					if ( _options.dumpDependencyInfo() )
-						_options.dumpDependency(Options::depArchive, archiveReader->path());
-					//<rdar://problem/17787306> -force_load_swift_libs
-					if (info.options.fForceLoad) {
-						archiveReader->forEachAtom(handler);
+					else if ( archiveReader != NULL ) {
+						_searchLibraries.push_back(LibraryInfo(archiveReader));
+						if ( _options.dumpDependencyInfo() )
+							_options.dumpDependency(Options::depArchive, archiveReader->path());
+						//<rdar://problem/17787306> -force_load_swift_libs
+						if (info.options.fForceLoad) {
+							archiveReader->forEachAtom(handler);
+						}
 					}
-				}
-				else {
-					throwf("linker option dylib at %s is not a dylib", info.path);
-				}
-			}
-			catch (const char* msg) {
-				warning("Auto-Linking supplied '%s', %s", info.path, msg);
-			}
+					else {
+						throwf("linker option dylib at %s is not a dylib", info.path);
+ 					}
+ 				}
+				catch (const char* msg) {
+					warning("Auto-Linking supplied '%s', %s", info.path, msg);
+ 				}
+ 			}
+			state.linkerOptionLibraries.insert(libName);
 		}
-		state.linkerOptionLibrariesProcessed.insert(libName);
 	}
 }
 
@@ -889,6 +906,7 @@ InputFiles::InputFiles(Options& opts, const char** archName)
 	pthread_mutex_init(&_parseLock, NULL);
 	pthread_cond_init(&_parseWorkReady, NULL);
 	pthread_cond_init(&_newFileAvailable, NULL);
+	_neededFileSlot = -1;
 #endif
 	const std::vector<Options::FileInfo>& files = _options.getInputFiles();
 	if ( files.size() == 0 )
@@ -986,7 +1004,7 @@ void InputFiles::parseWorkerThread() {
 			if (_s_logPThreads) printf("parsing index %u\n", slot);
 			try {
 				file = makeFile(entry, false);
-			} 
+			}
 			catch (const char *msg) {
 				if ( (strstr(msg, "architecture") != NULL) && !_options.errorOnOtherArchFiles() ) {
 					if ( _options.ignoreOtherArchInputFiles() ) {
@@ -1086,7 +1104,7 @@ ld::File* InputFiles::addDylib(ld::dylib::File* reader, const Options::FileInfo&
 
 	// log direct readers
 	if ( ! info.options.fIndirectDylib ) 
-		this->logDylib(reader, false);
+		this->logDylib(reader, false, false);
 
 	// update stats
 	_totalDylibsLoaded++;
@@ -1212,7 +1230,7 @@ void InputFiles::forEachInitialAtom(ld::File::AtomHandler& handler, ld::Internal
 			{
 				ld::archive::File* archive = (ld::archive::File*)file;
 				// <rdar://problem/9740166> force loaded archives should be in LD_TRACE
-				if ( (info.options.fForceLoad || _options.fullyLoadArchives()) && _options.traceArchives() ) 
+				if ( (info.options.fForceLoad || _options.fullyLoadArchives()) && (_options.traceArchives() || _options.traceEmitJSON()) )
 					logArchive(archive);
 				_searchLibraries.push_back(LibraryInfo(archive));
 				if ( _options.dumpDependencyInfo() )
@@ -1309,16 +1327,17 @@ bool InputFiles::searchLibraries(const char* name, bool searchDylibs, bool searc
                 ld::archive::File *archiveFile = lib.archive();
                 if ( dataSymbolOnly ) {
                     if ( archiveFile->justInTimeDataOnlyforEachAtom(name, handler) ) {
-                        if ( _options.traceArchives() ) 
+                        if ( _options.traceArchives() || _options.traceEmitJSON())
                             logArchive(archiveFile);
                         _options.snapshot().recordArchive(archiveFile->path());
+						// DALLAS _state.archives.push_back(archiveFile);
                         // found data definition in static library, done
                        return true;
                     }
                 }
                 else {
                     if ( archiveFile->justInTimeforEachAtom(name, handler) ) {
-                        if ( _options.traceArchives() ) 
+                        if ( _options.traceArchives() || _options.traceEmitJSON())
                             logArchive(archiveFile);
                         _options.snapshot().recordArchive(archiveFile->path());
                         // found definition in static library, done
@@ -1433,6 +1452,18 @@ void InputFiles::dylibs(ld::Internal& state)
 		}
 		// <rdar://problem/15002251> make implicit dylib order be deterministic by sorting by install_name
 		std::sort(implicitDylibs.begin(), implicitDylibs.end(), DylibByInstallNameSorter());
+
+		if ( _options.traceDylibs() ) {
+			for (ld::dylib::File* dylib :  implicitDylibs) {
+				if ( dylib->speculativelyLoaded() && !dylib->explicitlyLinked() && dylib->providedExportAtom() ) {
+					const char* fullPath = dylib->path();
+					char realName[MAXPATHLEN];
+					if ( realpath(fullPath, realName) != NULL )
+						fullPath = realName;
+					logTraceInfo("[Logging for XBS] Used dynamic library: %s\n", fullPath);
+				}
+			}
+		}
 		state.dylibs.insert(state.dylibs.end(), implicitDylibs.begin(), implicitDylibs.end());
 	}
 
@@ -1450,6 +1481,14 @@ void InputFiles::dylibs(ld::Internal& state)
 		throw "dynamic main executables must link with libSystem.dylib";
 }
 
+void InputFiles::archives(ld::Internal& state)
+{
+	for (const std::string path :  _archiveFilePaths) {
+		
+		state.archivePaths.push_back(path);
+	}
+}
+
 
 } // namespace tool 
 } // namespace ld 
