@@ -258,13 +258,15 @@ bool MachOChecker<arm64>::validFile(const uint8_t* fileContent)
 }
 #endif
 
+
+template <> uint8_t MachOChecker<ppc>::loadCommandSizeMask()	{ return 0x03; }
+template <> uint8_t MachOChecker<ppc64>::loadCommandSizeMask()	{ return 0x07; }
 template <> uint8_t MachOChecker<x86>::loadCommandSizeMask()	{ return 0x03; }
 template <> uint8_t MachOChecker<x86_64>::loadCommandSizeMask() { return 0x07; }
 template <> uint8_t MachOChecker<arm>::loadCommandSizeMask()	{ return 0x03; }
 template <> uint8_t MachOChecker<arm64>::loadCommandSizeMask()	{ return 0x07; }
 
 
-
 template <>
 x86::P::uint_t MachOChecker<x86>::getInitialStackPointer(const macho_thread_command<x86::P>* threadInfo)
 {
@@ -290,6 +292,13 @@ arm64::P::uint_t MachOChecker<arm64>::getInitialStackPointer(const macho_thread_
 }
 
 
+template <>
+ppc::P::uint_t MachOChecker<ppc>::getEntryPoint(const macho_thread_command<ppc::P>* threadInfo)
+{
+	return threadInfo->thread_register(0);
+}
+
+
 template <>
 x86::P::uint_t MachOChecker<x86>::getEntryPoint(const macho_thread_command<x86::P>* threadInfo)
 {
@@ -343,7 +352,6 @@ const char* MachOChecker<A>::archName()
 }
 
 
-
 template <typename A>
 MachOChecker<A>::MachOChecker(const uint8_t* fileContent, uint32_t fileLength, const char* path, const char* verifierDstRoot)
  : fHeader(NULL), fLength(fileLength), fInstallName(NULL), fStrings(NULL), fSymbols(NULL), fSymbolCount(0), fDynamicSymbolTable(NULL), fIndirectTableCount(0),
@@ -661,8 +669,12 @@ void MachOChecker<A>::checkLoadCommands()
 
 	// verify encryption info
 	if ( encryption_info != NULL ) {
-		if ( fHeader->filetype() != MH_EXECUTE )
-			throw "LC_ENCRYPTION_INFO load command is only legal in main executables";
+		switch ( fHeader->filetype() ) {
+			case MH_EXECUTE: case MH_DYLIB: case MH_BUNDLE: 
+				break;  // okay
+			default: 
+				throw "LC_ENCRYPTION_INFO load command is not allowed in this file type";
+		}
 		if ( encryption_info->cryptoff() <  (sizeof(macho_header<P>) + fHeader->sizeofcmds()) )
 			throw "LC_ENCRYPTION_INFO load command has cryptoff covers some load commands";
 		if ( (encryption_info->cryptoff() % 4096) != 0 )
@@ -712,8 +724,6 @@ void MachOChecker<A>::checkLoadCommands()
 						throw "string pool extends beyond __LINKEDIT";
 					if ( (symtab->stroff() % 4) != 0 ) // work around until rdar://problem/4737991 is fixed
 						throw "string pool start not pointer aligned";
-					if ( (symtab->strsize() % sizeof(pint_t)) != 0 )	
-						throw "string pool size not a multiple of pointer size";
 				}
 				break;
 			case LC_DYSYMTAB:
@@ -1131,6 +1141,7 @@ arm64::P::uint_t MachOChecker<arm64>::relocBase()
 	return fFirstWritableSegment->vmaddr();
 }
 
+
 template <typename A>
 bool MachOChecker<A>::addressInWritableSegment(pint_t address)
 {
@@ -1274,6 +1285,7 @@ void MachOChecker<arm64>::checkLocalReloation(const macho_relocation_info<P>* re
 }
 #endif
 
+
 template <typename A>
 void MachOChecker<A>::checkRelocations()
 {
